@@ -4,7 +4,7 @@ from itertools import count
 
 from .common import maybe_declare
 from .compression import compress
-from .connection import maybe_channel, is_connection
+from .connection import is_connection, maybe_channel
 from .entity import Exchange, Queue, maybe_delivery_mode
 from .exceptions import ContentDisallowed
 from .serialization import dumps, prepare_accept_content
@@ -115,7 +115,7 @@ class Producer:
                 mandatory=False, immediate=False, priority=0,
                 content_type=None, content_encoding=None, serializer=None,
                 headers=None, compression=None, exchange=None, retry=False,
-                retry_policy=None, declare=None, expiration=None,
+                retry_policy=None, declare=None, expiration=None, timeout=None,
                 **properties):
         """Publish message to the specified exchange.
 
@@ -144,6 +144,8 @@ class Producer:
                 supported by :meth:`~kombu.Connection.ensure`.
             expiration (float): A TTL in seconds can be specified per message.
                 Default is no expiration.
+            timeout (float): Set timeout to wait maximum timeout second
+                for message to publish.
             **properties (Any): Additional message properties, see AMQP spec.
         """
         _publish = self._publish
@@ -175,12 +177,12 @@ class Producer:
         return _publish(
             body, priority, content_type, content_encoding,
             headers, properties, routing_key, mandatory, immediate,
-            exchange_name, declare,
+            exchange_name, declare, timeout
         )
 
     def _publish(self, body, priority, content_type, content_encoding,
                  headers, properties, routing_key, mandatory,
-                 immediate, exchange, declare):
+                 immediate, exchange, declare, timeout=None):
         channel = self.channel
         message = channel.prepare_message(
             body, priority, content_type,
@@ -198,6 +200,7 @@ class Producer:
             message,
             exchange=exchange, routing_key=routing_key,
             mandatory=mandatory, immediate=immediate,
+            timeout=timeout
         )
 
     def _get_channel(self):
@@ -211,6 +214,7 @@ class Producer:
 
     def _set_channel(self, channel):
         self._channel = channel
+
     channel = property(_get_channel, _set_channel)
 
     def revive(self, channel):
@@ -237,6 +241,7 @@ class Producer:
 
     def release(self):
         pass
+
     close = release
 
     def _prepare(self, body, serializer=None, content_type=None,
@@ -358,7 +363,7 @@ class Consumer:
     #: Mapping of queues we consume from.
     _queues = None
 
-    _tags = count(1)   # global
+    _tags = count(1)  # global
 
     def __init__(self, channel, queues=None, no_ack=None, auto_declare=None,
                  callbacks=None, on_decode_error=None, on_message=None,
@@ -483,6 +488,7 @@ class Consumer:
         for tag in self._active_tags.values():
             cancel(tag)
         self._active_tags.clear()
+
     close = cancel
 
     def cancel_by_queue(self, queue):
@@ -620,7 +626,7 @@ class Consumer:
             return on_m(message) if on_m else self.receive(decoded, message)
 
     def __repr__(self):
-        return '<{name}: {0.queues}>'.format(self, name=type(self).__name__)
+        return f'<{type(self).__name__}: {self.queues}>'
 
     @property
     def connection(self):
